
require 'test/unit'
require 'scriptroute'
require 'scriptroute/nameify'
require 'ipaddr'

class ScriptrouteTest < Test::Unit::TestCase
  def test_abba # be first
    unless Scriptroute.is_daemon_running? 
      exit
    end
    
    Scriptroute::DaemonVersion()

    p = Scriptroute::ICMPecho.new
    p.ip_dst = '128.8.126.104'
    responses = Scriptroute::send_train( [ [ 0, p.marshal ] ] )
    p.ip_dst = Scriptroute::dns_lookup('kohoutek.cs.umd.edu')
    responses = Scriptroute::send_train( [ [ 0, p.marshal ] ] )
    # puts responses.inspect
    puts responses[0].rtt
  end
  def test_abc_get_config
    raise "no config returned" unless Scriptroute::DaemonConfig() 
    raise "config not a hash" unless Scriptroute::DaemonConfig().is_a?(Hash)
    raise "config lacks an expected key" unless Scriptroute::DaemonConfig().has_key?('Filter.listen_filter')
  end
  def test_ally
    verdict = Scriptroute::Ally.new('128.8.126.92', '128.8.126.104')
    puts verdict
  end
  def test_rockettrace
    result = Scriptroute::Rockettrace.new('128.8.126.104', 1, false, 2, 64, 4)
    puts result
  end

  def test_forbidden
    $stderr.puts "the next line should be 'ERROR: packet administratively filtered'..."
    i = Scriptroute::ICMPecho.new(0)
    # an address ending with .255 seems more likely to be broadcast. 
    i.ip_dst = 0x805f02ff 
    j = Scriptroute::ICMPecho.new(0)
    # an address ending with .255 seems more likely to be broadcast. 
    j.ip_dst = '128.8.126.104'
    timeout(60) do
      p = Scriptroute::send_train([ 
          Struct::DelayedPacket.new(0, Scriptroute::pkt_from_string(i.marshal) ),
          Struct::DelayedPacket.new(0, Scriptroute::pkt_from_string(j.marshal) )
        ])
      puts p[0].probe
      puts p[0].response
    end
  rescue TimeoutError
    puts "timed out trying to complete a send_train with a forbidden destination"
    false
  rescue Scriptroute::ScriptrouteError
    true
  end
    
  def test_packets_with_daemon
    i = Scriptroute::ICMPecho.new(0)
    i.ip_dst = 0x805f0218 # poplar
    # i.ip_dst = 0x84ef3314 # www.cs.ucsd.edu often unresponsive
    i.ip_dst = 0xc6ca4b65 # www.sdsc.edu
    # i.ip_ttl=8
    # rr = RecordRoute_option.new
    rr = Scriptroute::Timestamp_option.new(1)
    i.add_option(rr)
    # i.uh_dport = 3000
    m = i.marshal
    has_bytes = m.methods.include?(:bytes)
      puts m.length.to_s + " " + m.bytes.map { |b| "%x " % b }.join if has_bytes
      # old ruby.
      puts m.length.to_s + " " + m.gsub(/./) { |b| "%x " % b[0].to_i }
    
    p = Scriptroute::send_train([ Struct::DelayedPacket.new(0,
          Scriptroute::pkt_from_string(i.marshal) ) ])
    puts p[0].probe
    puts p[0].response
    
    if(p[0].response && p[0].response.packet.icmp_type != Scriptroute::ICMP::ICMP_PARAMETERPROB) then
      puts "****************** (%d)" % p[0].response.packet.icmp_type
    end
    
  end
  def test_packets_without_daemon
    m = Scriptroute::ICMPecho.new.marshal
    has_bytes = m.methods.include?(:bytes)
    puts m.length.to_s + " " + m.gsub(/./) { |b| "%x " % b[0].to_i }
    puts m.length.to_s + " " + m.bytes.map { |b| "%x " % b }.join if has_bytes ## 1.9
    # puts "decode: " + Scriptroute::stringpacket(m)
    q = Scriptroute::pkt_from_string(m)
    
    m = Scriptroute::ICMPtstamp.new.marshal
    puts m.length.to_s + " " + m.gsub(/./) { |b| "%x " % b[0].to_i }
    puts m.length.to_s + " " + m.bytes.map { |b| "%x " % b }.join if has_bytes ## 1.9
    #  puts "decode: " + Scriptroute::stringpacket(m)
    q = Scriptroute::pkt_from_string(m)
    # garbage. puts "encode: " + Scriptroute::pkt_from_string(m).to_s
    
    m = Scriptroute::TCP.new.marshal
    puts m.length.to_s + " " + m.gsub(/./) { |b| "%x " % b[0].to_i }
    puts m.length.to_s + " " + m.bytes.map { |b| "%x " % b }.join if has_bytes ## 1.9
    #  puts "decode: " + Scriptroute::stringpacket(m)
    # garbage. puts "encode: " + Scriptroute::pkt_from_string(m).to_s
    q = Scriptroute::pkt_from_string(m)
    
    m = Scriptroute::UDP.new.marshal
    puts m.length.to_s + " " + m.gsub(/./) { |b| "%x " % b[0].to_i }
    puts m.length.to_s + " " + m.bytes.map { |b| "%x " % b }.join if has_bytes ## 1.9
    #  puts "decode: " + Scriptroute::stringpacket(m)
    # garbage. puts "encode: " + Scriptroute::pkt_from_string(m).to_s
    q = Scriptroute::pkt_from_string(m)

    m = Scriptroute::UDP.new(8).marshal
    puts m.length.to_s + " " + m.gsub(/./) { |b| "%x " % b[0].to_i }
    puts m.length.to_s + " " + m.bytes.map { |b| "%x " % b }.join if has_bytes ## 1.9
    # garbage. puts "decode: " + Scriptroute::IPv4.creator(m).to_s
    q = Scriptroute::pkt_from_string(m)
    # garbage. puts "encode: " + Scriptroute::pkt_from_string(m).to_s

    t = Scriptroute::TCP.new
    opt = Scriptroute::Timestamp_option.new(1)
    optr = Scriptroute::RecordRoute_option.new
    t.add_option(optr)
    t.ip_dst = 0x805f0218
    t.ip_ttl = 3
    t.th_seq = 0x01020304
    t.th_ack = 0x09080706
    m=t.marshal
    puts m.length.to_s + " " + m.gsub(/./) { |b| "%x " % b[0].to_i }
    puts m.length.to_s + " " + m.bytes.map { |b| "%x " % b }.join if has_bytes ## 1.9
    # puts "decode: " + Scriptroute::stringpacket(m)
    # garbage. puts "encode: " + Scriptroute::pkt_from_string(m).to_s
    q = Scriptroute::pkt_from_string(m)
    
    #1.times { |i|
    #  m=t.marshal
    #  p = Scriptroute::send_train([ Struct::DelayedPacket.new(0, Scriptroute::pkt_from_string(m) ) ])
    #  puts p[0].probe
    #  puts p[0].response
    #  if(p[0].response && p[0].response.packet.icmp_type != ICMP_PARAMETERPROB) then
    #    puts "****************** (%d)" % p[0].response.packet.icmp_type
    #  end
    #  sleep(1)
    #}
    
    puts "--"
    puts "No exceptions means the packets.rb self test probably worked okay. :)"
    puts " Specifically, any bad checksum errors are because the interpreter doesn't"
    puts " calculate them."
    puts "--"
  end

  def test_nameify
    puts Scriptroute.nameify("12.123.203.170")
  end

  # pretty much checks whether the packet gets filled in.
  # manually constructed with pack, rather than via
  # scriptroute/packets
  def test_v6
    daddr = IPAddr.new "ff02::fb" # "fd00::230:67ff:febb:7d18"
    # daddr = IPAddr.new "fe80::60c:ceff:fee4:a62" # wlp
    daddr = IPAddr.new "fd00::60c:ceff:fee4:a62" # wlp
    saddr = IPAddr.new "::0"
#     saddr = IPAddr.new "fd00::230:67ff:febb:7d18"   # zar
    # saddr = IPAddr.new "fe80::230:67ff:febb:7d18" # fd00::230:67ff:febb:7d18"
    payload_len = 40 # 

    # hard coded the cksum.
    icmp = [ Scriptroute::ICMP6::ICMP6_ECHO, 0, 0xc12b, 0, 1 ].pack("ccnnn") + ( "\0" * 56 )
    v6packet = [ 6 << 4, 0, 0, icmp.length, Scriptroute::IPv6::IPPROTO_ICMP6, 
      64, saddr.hton, daddr.hton ].pack("ccnncca16a16") + icmp

    raise "too short (#{v6packet.length})" unless v6packet.length >= 40 
    puts v6packet.length
    puts v6packet.unpack("H*")
      
    p = Scriptroute::send_train([ Struct::DelayedPacket.new(0, v6packet) ])
    puts p
    
    p[0].probe or raise "couldn't manage sending a v6 probe"
    p[0].probe.packet or raise "couldn't manage sending a v6 probe packet"
    p[0].probe.packet.ip6_src or raise "v6 probe packet lacks a source"
    p[0].probe.packet.ip6_src != IPAddr.new("::0") or raise "v6 probe packet source seems to be unset."
    p[0].probe.packet.icmp_cksum != 0 or raise "v6 probe packet 99.99% likely unset checksum."
  end

  # tries to build an medium level of easiness ipv6 ping.
  def test_v6_lookup
    daddr = nil
    Resolv::DNS.open do |dns|
      ress = dns.getresources "www.google.com", Resolv::DNS::Resource::IN::AAAA
      ress.each { |r| daddr = r.address.to_s }
    end
    raise "no address" unless daddr
    p = Scriptroute::ICMP6echo.new(20)
    p.ip6_dst = daddr
    begin 
      r = Scriptroute::send_train([ Struct::DelayedPacket.new(0, p) ])
      puts r
    rescue Scriptroute::ScriptrouteError => e
      if e.to_s =~ /unreachable/ then
        puts "Expected ERROR when host supports v6 but lacks routes to the outside world."
      else
        puts "rescued unexpected error: " + e.to_s
      end
    end  
  end
  def test_v6_localhost
    p = Scriptroute::ICMP6echo.new(20)
    p.ip6_dst = IPAddr.new("::1")
    p.ip6_dst = "::1"
    r = Scriptroute::send_train([ Struct::DelayedPacket.new(0, p) ])
  rescue Scriptroute::ScriptrouteError => e
    puts "Expected ERROR because localhost is filtered."
  end
  def test_v6_implicit_lookup
    p = Scriptroute::UDP6.new(20)
    p.ip6_dst = "www.google.com"
    puts p.ip6_dst
  end

  # I don't know whether I want to be able to send to all-routesr 
  def test_v6_mcast
    failure = false
    begin
      p = Scriptroute::ICMP6echo.new(20)
      p.ip6_dst = IPAddr.new("ff02::1")  # all nodes.
      r = Scriptroute::send_train([ Struct::DelayedPacket.new(0, p) ])
      failure = true if r[0].probe 
    rescue Scriptroute::ScriptrouteError => e
      puts "Expected ERROR because multicast is filtered or source cannot be set."
    end
    begin
      p = Scriptroute::ICMP6echo.new(20)
      p.ip6_dst = IPAddr.new("ff02::2")  # all routers.
      r = Scriptroute::send_train([ Struct::DelayedPacket.new(0, p) ])
      failure = true if r[0].probe 
    rescue Scriptroute::ScriptrouteError => e
      puts "Expected ERROR because multicast is filtered or source cannot be set."
    end
    failure
  end

  def test_v6_onehop
    daddr = nil
    Resolv::DNS.open do |dns|
      ress = dns.getresources "www.google.com", Resolv::DNS::Resource::IN::AAAA
      ress.each { |r| daddr = r.address.to_s }
    end
    raise "no address" unless daddr
    p = Scriptroute::ICMP6echo.new(20)
    p.ip6_dst = daddr
    p.ip6_hlim = 1
    begin 
      r = Scriptroute::send_train([ Struct::DelayedPacket.new(0, p) ])
      if r[0].response then
        puts "success from first hop: %s:" % r[0].response.packet.ip6_src
        puts r[0].response.packet
        true
      else
        puts "failed: expected to see a response to the one-hop ttl limited probe"
        false
      end
    rescue Scriptroute::ScriptrouteError => e
      if e.to_s =~ /unreachable/ then
        puts "Expected ERROR when host supports v6 but lacks routes to the outside world."
      else
        puts "rescued unexpected error: " + e.to_s
      end
    end  
  end

  def test_v6_udp_tr
    daddr = nil
    Resolv::DNS.open do |dns|
      ress = dns.getresources "www.google.com", Resolv::DNS::Resource::IN::AAAA
      ress.each { |r| daddr = r.address.to_s }
    end
    raise "no address" unless daddr
    p = Scriptroute::UDP6.new(20)
    p.ip6_dst = daddr
    puts p.to_s
    begin 
      [4,5,6,20].each { |ttl|
        p.ip6_hlim = ttl
        r = Scriptroute::send_train([ Struct::DelayedPacket.new(0, p) ])
        if r[0].response then
          puts "success from hop %d: %s:" % [ ttl, r[0].response.packet.ip6_src ]
          puts r[0].response.packet
          true
        else
          puts "failed: expected to see a response to the one-hop ttl limited probe"
          false
        end
      }
    rescue Scriptroute::ScriptrouteError => e
      if e.to_s =~ /unreachable/ then
        puts "Expected ERROR when host supports v6 but lacks routes to the outside world."
      else
        puts "rescued unexpected error: " + e.to_s
      end
    end
  end  
end

# Local Variables:
# compile-command: "ruby -I ../lib ./test_scriptroute.rb"
# End:
